@@ -49,7 +49,6 @@ int ci_insert_line2(struct channel *c, int pos, const char *str, int len);
 int co_inject(struct channel *chn, const char *msg, int len);
 int co_getchar(const struct channel *chn, char *c);
 int co_getline(const struct channel *chn, char *str, int len);
-int co_getdelim(const struct channel *chn, char *str, int len, const char *delim, char escape);
 int co_getword(const struct channel *chn, char *str, int len, char sep);
 int co_getblk(const struct channel *chn, char *blk, int len, int offset);
 int co_getline_nc(const struct channel *chn, const char **blk1, size_t *len1, const char **blk2, size_t *len2);
@@ -836,27 +835,19 @@ static inline int ci_space_for_replace(const struct channel *chn)
 	return end - ci_tail(chn);
 }
 
-/* Allocates a buffer for channel <chn>, but only if it's guaranteed that it's
- * not the last available buffer or it's the response buffer. Unless the buffer
- * is the response buffer, an extra control is made so that we always keep
- * <tune.buffers.reserved> buffers available after this allocation. Returns 0 in
- * case of failure, non-zero otherwise.
+/* Allocates a buffer for channel <chn>. Returns 0 in case of failure, non-zero
+ * otherwise.
  *
  * If no buffer are available, the requester, represented by <wait> pointer,
  * will be added in the list of objects waiting for an available buffer.
  */
 static inline int channel_alloc_buffer(struct channel *chn, struct buffer_wait *wait)
 {
-	int margin = 0;
-
-	if (!(chn->flags & CF_ISRESP))
-		margin = global.tune.reserved_bufs;
-
-	if (b_alloc_margin(&chn->buf, margin) != NULL)
+	if (b_alloc(&chn->buf) != NULL)
 		return 1;
 
-	if (!LIST_ADDED(&wait->list))
-		LIST_ADDQ(&ti->buffer_wq, &wait->list);
+	if (!LIST_INLIST(&wait->list))
+		LIST_APPEND(&ti->buffer_wq, &wait->list);
 
 	return 0;
 }
@@ -937,36 +928,26 @@ static inline int32_t channel_htx_fwd_headers(struct channel *chn, struct htx *h
  * when data have been read directly from the buffer. It is illegal to call
  * this function with <len> causing a wrapping at the end of the buffer. It's
  * the caller's responsibility to ensure that <len> is never larger than
- * chn->o. Channel flags WRITE_PARTIAL and WROTE_DATA are set.
+ * chn->o.
  */
 static inline void co_skip(struct channel *chn, int len)
 {
 	b_del(&chn->buf, len);
 	chn->output -= len;
 	c_realign_if_empty(chn);
-
-	/* notify that some data was written to the SI from the buffer */
-	chn->flags |= CF_WRITE_PARTIAL | CF_WROTE_DATA;
-	chn_prod(chn)->flags &= ~SI_FL_RXBLK_ROOM; // si_rx_room_rdy()
 }
 
 /* HTX version of co_skip(). This function skips at most <len> bytes from the
  * output of the channel <chn>. Depending on how data are stored in <htx> less
- * than <len> bytes can be skipped. Channel flags WRITE_PARTIAL and WROTE_DATA
- * are set.
+ * than <len> bytes can be skipped..
  */
 static inline void co_htx_skip(struct channel *chn, struct htx *htx, int len)
 {
 	struct htx_ret htxret;
 
 	htxret = htx_drain(htx, len);
-	if (htxret.ret) {
+	if (htxret.ret)
 		chn->output -= htxret.ret;
-
-		/* notify that some data was written to the SI from the buffer */
-		chn->flags |= CF_WRITE_PARTIAL | CF_WROTE_DATA;
-		chn_prod(chn)->flags &= ~SI_FL_RXBLK_ROOM; // si_rx_room_rdy()
-	}
 }
 
 /* Tries to copy chunk <chunk> into the channel's buffer after length controls.
