@@ -667,7 +667,7 @@ int sock_conn_check(struct connection *conn)
 	if (!(conn->flags & CO_FL_WAIT_L4_CONN))
 		return 1; /* strange we were called while ready */
 
-	if (!fd_send_ready(fd))
+	if (!fd_send_ready(fd) && !(fdtab[fd].state & (FD_POLL_ERR|FD_POLL_HUP)))
 		return 0;
 
 	/* Here we have 2 cases :
@@ -835,7 +835,7 @@ int sock_drain(struct connection *conn)
 	if (fdtab[fd].state & (FD_POLL_ERR|FD_POLL_HUP))
 		goto shut;
 
-	if (!fd_recv_ready(fd))
+	if (!(conn->flags & CO_FL_WANT_DRAIN) && !fd_recv_ready(fd))
 		return 0;
 
 	/* no drain function defined, use the generic one */
